#include "test.h"

char buff[1000000];
char buff2[100000];

DEFINE_TEST(test_read_extract)
{
	struct archive_entry *ae;
	struct archive *a;
	struct stat st;
	ssize_t used;
	int i;
	char tmpdir[] = "/tmp/test-read_extract.XXXXXX";

	/* Create a temp directory for the following tests. */
	mkdtemp(tmpdir);
	chdir(tmpdir);
	/* Force the umask to something predictable. */
	umask(022);

	/* Create a new archive in memory containing various types of entries. */
	assert((a = archive_write_new()) != NULL);
	assertA(0 == archive_write_set_format_ustar(a));
	assertA(0 == archive_write_set_compression_none(a));
	assertA(0 == archive_write_open_memory(a, buff, sizeof(buff), &used));
	/* A regular file. */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "file");
	archive_entry_set_mode(ae, S_IFREG | 0755);
	for (i = 0; i < sizeof(buff2); i++)
		buff2[i] = (unsigned char)rand();
	archive_entry_set_size(ae, sizeof(buff2));
	assertA(0 == archive_write_header(a, ae));
	assertA(sizeof(buff2) == archive_write_data(a, buff2, sizeof(buff2)));
	archive_entry_free(ae);
	/* A directory (not writable). */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "dir");
	archive_entry_set_mode(ae, S_IFDIR | 0555);
	assertA(0 == archive_write_header(a, ae));
	/* A file in the unwritable directory. */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "dir/file");
	archive_entry_set_mode(ae, S_IFREG | 0700);
	assertA(0 == archive_write_header(a, ae));
	/* A file in a dir that is not already in the archive. */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "dir2/file");
	archive_entry_set_mode(ae, S_IFREG | 0000);
	assertA(0 == archive_write_header(a, ae));
	/* A dir with a trailing /. */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "dir3/.");
	archive_entry_set_mode(ae, S_IFDIR | 0710);
	assertA(0 == archive_write_header(a, ae));
	/* Multiple dirs with a single entry. */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "dir4/a/../b/../c/");
	archive_entry_set_mode(ae, S_IFDIR | 0711);
	assertA(0 == archive_write_header(a, ae));
	/* A symlink. */
	assert((ae = archive_entry_new()) != NULL);
	archive_entry_copy_pathname(ae, "symlink");
	archive_entry_set_mode(ae, S_IFLNK | 0755);
	archive_entry_set_symlink(ae, "file");
	assertA(0 == archive_write_header(a, ae));
	/* Close out the archive. */
	assertA(0 == archive_write_close(a));
#if ARCHIVE_API_VERSION > 1
	assertA(0 == archive_write_finish(a));
#else
	archive_write_finish(a);
#endif

	/* Extract the entries to disk. */
	assert((a = archive_read_new()) != NULL);
	assertA(0 == archive_read_support_format_all(a));
	assertA(0 == archive_read_support_compression_all(a));
	assertA(0 == archive_read_open_memory(a, buff, sizeof(buff)));
	for (i = 7; i > 0; i--) {
		assertA(0 == archive_read_next_header(a, &ae));
		assertA(0 == archive_read_extract(a, ae, 0));
	}
	assertA(ARCHIVE_EOF == archive_read_next_header(a, &ae));
	assert(0 == archive_read_close(a));
#if ARCHIVE_API_VERSION > 1
	assert(0 == archive_read_finish(a));
#else
	archive_read_finish(a);
#endif

	/* Test the entries on disk. */
	assert(0 == stat("file", &st));
	assert(st.st_mode == (S_IFREG | 0755));
	assert(0 == stat("dir", &st));
	assert(st.st_mode == (S_IFDIR | 0555));
	assert(0 == stat("dir/file", &st));
	assert(st.st_mode == (S_IFREG | 0700));
	assert(0 == stat("dir2", &st));
	assert(st.st_mode == (S_IFDIR | 0755));
	assert(0 == stat("dir2/file", &st));
	assert(st.st_mode == (S_IFREG | 0000));
	assert(0 == stat("dir3", &st));
	assert(st.st_mode == (S_IFDIR | 0710));
	assert(0 == stat("dir4", &st));
	assert(st.st_mode == (S_IFDIR | 0755));
	assert(0 == stat("dir4/a", &st));
	assert(st.st_mode == (S_IFDIR | 0755));
	assert(0 == stat("dir4/b", &st));
	assert(st.st_mode == (S_IFDIR | 0755));
	assert(0 == stat("dir4/c", &st));
	assert(st.st_mode == (S_IFDIR | 0711));
	assert(0 == lstat("symlink", &st));
	assert(st.st_mode == (S_IFLNK | 0755));
	assert(0 == stat("symlink", &st));
	assert(st.st_mode == (S_IFREG | 0755));
}
